clear all
close all

Fs=14; % font size
thresh = 20; % mm, threshold for outliers

ref = cellstr('xshos'); % reference site

f = './gps.dat';

% cGPS stations:
T=readtable([f],'Delimiter',' ','ReadVariableNames',0);
C=table2cell(T);
wid=length(C(1,:));
N=cell2mat(C(:,1:2)); % coordinates
L=C(:,3);             % station labels

% now get all of the sites:
for i=1:length(L(:))
  xsite=char(L(i));
  site=xsite(2:5);
  gvar=['n' site];  % displ. data
  x(i)=N(i,1);
  y(i)=N(i,2);
  gfile=[gvar '.dat'];

   if ((exist(gfile, 'file')) == 2)
    load(gfile);
    z=eval(gvar);
    t=z(:,1);
    e=z(:,3); 
    n=z(:,2); 
% find common data with ref. site

    [pe,se]=polyfit(t,e,1);
    [pn,sn]=polyfit(t,n,1);
    yn=pn(2)+pn(1)*t;
    ye=pe(2)+pe(1)*t;

    vn(i)=pn(1);
    ve(i)=pe(1);

   end
end

vn=vn-vn(length(vn)-3);
ve=ve-ve(length(vn)-3);

orig=[-117.5 35.5];
xo=orig(1); yo=orig(2);
[xo,yo]=utm2ll(xo,yo,0,1);
[xx,yy]=utm2ll(x(:),y(:),0,1);
xu=reshape((xx-xo)*1e-3,size(x)); 
yu=reshape((yy-yo)*1e-3,size(x));
llrat=[1 1 1];

xmin=min(xu(:)); xmax=max(xu(:)); ymin=min(yu(:)); ymax=max(yu(:));

tri = delaunay(xu,yu);
figure
%triplot(tri,xu,yu), hold on
for i=1:length(xu(:))
 xsite=char(L(i));
 site=xsite(2:5);
 text(xu(i),yu(i),site), hold on
end
quiver(xu,yu,ve,vn,0);
%return

% set up a system of linear equations for strain and rotation rates
R=length(tri(:,1)); % number of resolution cells (Delaunay triangles)
F=3*R+1; % number of degrees of freedom
d=zeros(4*R,1);
A=zeros(4*R,F);
for i=1:length(tri(:,1)) % for each triangle:
 dx1 = xu(tri(i,2))-xu(tri(i,1));
 dy1 = yu(tri(i,2))-yu(tri(i,1));
 dx2 = xu(tri(i,3))-xu(tri(i,1));
 dy2 = yu(tri(i,3))-yu(tri(i,1));
 vx1 = ve(tri(i,2))-ve(tri(i,1));
 vy1 = vn(tri(i,2))-vn(tri(i,1));
 vx2 = ve(tri(i,3))-ve(tri(i,1));
 vy2 = vn(tri(i,3))-vn(tri(i,1));
 d((i-1)*4+1)=vx1; d((i-1)*4+2)=vy1;
 d((i-1)*4+3)=vx2; d((i-1)*4+4)=vy2;

 A((i-1)*4+1,(i-1)*3+1)=dx1;
 A((i-1)*4+1,(i-1)*3+2)=dy1;
 A((i-1)*4+1,F)=dy1;

 A((i-1)*4+2,(i-1)*3+2)=dx1;
 A((i-1)*4+2,(i-1)*3+3)=dy1;
 A((i-1)*4+2,F)=-dx1;

 A((i-1)*4+3,(i-1)*3+1)=dx2;
 A((i-1)*4+3,(i-1)*3+2)=dy2;
 A((i-1)*4+3,F)=dy2;
 
 A((i-1)*4+4,(i-1)*3+2)=dx2;
 A((i-1)*4+4,(i-1)*3+3)=dy2;
 A((i-1)*4+4,F)=-dx2;

end

%invert:
u=A\d;

%return
w=u(F) % rotation rate
L=10; % length of tickmarks

for i=1:length(tri(:,1))
 e11(i)=u((i-1)*3+1);
 e12(i)=u((i-1)*3+2);
 e22(i)=u((i-1)*3+3);
 theta(i)=0.5*atan(e12(i)/(e11(i)-e22(i)))+pi/2;
 
 xv = [xu(tri(i,1)) xu(tri(i,2)) xu(tri(i,3))]; % x coord of vertixes
 yv = [yu(tri(i,1)) yu(tri(i,2)) yu(tri(i,3))]; % y coord of vertixes
 x_centroid(i) = mean(xv);
 y_centroid(i) = mean(yv);
 dx = [-1 1]*L*cos(theta(i))+x_centroid(i);
 dy = [-1 1]*L*sin(theta(i))+y_centroid(i);
 
end

fname = 'p_strain.dat';
arr = [x_centroid' y_centroid' (pi/2-theta')*180/pi];
save(fname,'arr','-ascii');

[xq,yq] = meshgrid(-20:.5:50, -40:.5:40);
vq = griddata(x_centroid,y_centroid,theta*180/pi,xq,yq,'v4');
figure
pcolor(xq,yq,vq), shading flat
colorbar

xq=0;
yq=30;
vq = griddata(x_centroid,y_centroid,theta*180/pi,xq,yq,'v4')
return

[xq,yq] = meshgrid(-20:4:20, 0:4:40);
e11q = griddata(x_centroid,y_centroid,e11,xq,yq,'v4');
e12q = griddata(x_centroid,y_centroid,e12,xq,yq,'v4');
e22q = griddata(x_centroid,y_centroid,e22,xq,yq,'v4');

figure
pcolor(xq,yq,e11q), shading flat
colorbar
title('e11')

figure
pcolor(xq,yq,e22q), shading flat
colorbar
title('e22')

figure
pcolor(xq,yq,e12q), shading flat
colorbar
title('e12')

figure
pcolor(xq,yq,e11q+e22q), shading flat
colorbar
title('dilatation')

figure
pcolor(xq,yq,0.5*sqrt((e11q-e22q).^2+e12q.^2)), shading flat
colorbar
title('max shear strain')

%e11q=1e-1;
%e22q=-1e-1;
%e12q=0e-1;
u1 = e11q.*xq+e12q.*yq;
u2=e12q.*xq+e22q.*yq;
A=1e1;

figure
quiver(xq,yq,A*u1,A*u2,0), hold on
axis('equal');

xq=-10;
yq=25;
e11q = griddata(x_centroid,y_centroid,e11,xq,yq,'v4');
e12q = griddata(x_centroid,y_centroid,e12,xq,yq,'v4');
e22q = griddata(x_centroid,y_centroid,e22,xq,yq,'v4');

xq=[-10:0.5:0];
yq=3*xq+30;

u1 = e11q.*xq+e12q.*yq;
u2=e12q.*xq+e22q.*yq;

plot(xq,yq,'r-'), hold on
plot(xq+A*u1,yq+A*u2,'b-'), hold on
axis('equal');

