clear all
close all

sig1=85*pi/180;  % initial orientation of sigma_1
mu=0.6; % coefficient of friction 
time_int=0:1:10; % time increment, Ma

Fs=14;
thresh = 20; % mm, threshold for outliers

ref = cellstr('xshos'); % reference site

f = './gps_sel.dat';

% cGPS stations:
T=readtable([f],'Delimiter',' ','ReadVariableNames',0);
C=table2cell(T);
wid=length(C(1,:));
N=cell2mat(C(:,1:2)); % coordinates
L=C(:,3);             % station labels

% now get all of the sites:
for i=1:length(L(:))
  xsite=char(L(i));
  site=xsite(2:5);
  gvar=['n' site];  % displ. data
  x(i)=N(i,1);
  y(i)=N(i,2);
  gfile=[gvar '.dat'];

   if ((exist(gfile, 'file')) == 2)
    load(gfile);
    z=eval(gvar);
    t=z(:,1);
    e=z(:,2); 
    n=z(:,3); 
    [pe,se]=polyfit(t,e,1);
    [pn,sn]=polyfit(t,n,1);
    yn=pn(2)+pn(1)*t;
    ye=pe(2)+pe(1)*t;

    vn(i)=pn(1);
    ve(i)=pe(1);

   end
end

vn=vn-vn(1);
ve=ve-ve(1);

orig=[-117.5 35.5];
xo=orig(1); yo=orig(2);
[xo,yo]=utm2ll(xo,yo,0,1);
[xx,yy]=utm2ll(x(:),y(:),0,1);
xu=reshape((xx-xo)*1e-3,size(x)); 
yu=reshape((yy-yo)*1e-3,size(x));
llrat=[1 1 1];
X1=[]; X2=[]; Y1=[]; Y2=[];

xmin=min(xu(:)); xmax=max(xu(:)); ymin=min(yu(:)); ymax=max(yu(:));

tri = delaunay(xu,yu);
figure(1)
for i=1:length(xu(:))
 xsite=char(L(i));
 site=xsite(2:5);
 text(xu(i),yu(i),site), hold on
end
quiver(xu,yu,ve,vn,0);

L=10; % length of tickmarks


[xq,yq] = meshgrid(-20:0.5:50, -40:0.5:40);
veq = griddata(xu,yu,ve,xq,yq,'v4');
vnq = griddata(xu,yu,vn,xq,yq,'v4');

figure(2)
pcolor(xq,yq,veq), shading flat
colorbar

opt=atan(1/mu)/2;
xq=[-5:0.25:5]; % x-coord
A=1; % a scaling factor 
xq2=[-25:0.5:25]; % x-coord


figure(3)

for i=1:length(time_int)

 theta1=sig1-opt; % "right plane"

 yq=tan(theta1)*xq+30;
 yq1=tan(sig1)*xq+30;
 veq = griddata(xu,yu,ve,xq,yq,'v4');
 vnq = griddata(xu,yu,vn,xq,yq,'v4');
 plot(xq,yq1,'k:'), hold on
 dt=time_int(i);
 ue=veq*dt;
 un=vnq*dt;
 ue=ue-ue(1);
 un=un-un(1);

 x1=xq+A*ue; y1=yq+A*un;
 plot(x1,y1,'bo'), hold on
 [Err, p1] = fit_2D_data(x1, y1, 'no');
 x1=xq+A*ue; y1=yq+A*un;

 yb=p1(1)*xq2+30;
 X1=[X1 xq2'];
 Y1=[Y1 yb'];

 plot(xq2,yb,'b-'), hold on
 axis('equal'), hold on

 theta1=sig1+opt; % "left plane"

 yq=tan(theta1)*xq+30;
 veq = griddata(xu,yu,ve,xq,yq,'v4');
 vnq = griddata(xu,yu,vn,xq,yq,'v4');
 ue=veq*dt;
 un=vnq*dt;
 ue=ue-ue(1);
 un=un-un(1);
 x1=xq+A*ue; y1=yq+A*un;
 plot(x1,y1,'ro'), hold on
 [Err, p2] = fit_2D_data(x1, y1, 'no');
 yb=p2(1)*xq2+30;
 plot(xq2,yb,'r--'), hold on
 X2=[X2 xq2'];
 Y2=[Y2 yb'];
 axis('equal'), hold on
 ang2 = atan((p2(1)-p1(1))/(1+ p1(1)*p2(1)))/pi*180;
 if ang2 < 0, ang2=ang2+180; end

 rot_ang(i)=ang2;
 xlabel('Eastings, km'), hold on
 ylabel('Northings, km'), hold on
 set(gca,'xlim',[-15 15],'ylim',[15 45]), hold on
 set(gca,'box','on'), hold on
 set(gca,'FontSize',Fs), hold on
end
xlabel('Eastings, km'), hold on
ylabel('Northings, km'), hold on
set(gca,'xlim',[-10 10],'ylim',[20 40]), hold on
set(gca,'box','on'), hold on
set(gca,'FontSize',Fs), hold on


save sigma1 xq yq1;
save rot0_6 time_int rot_ang;
save rot_lines X1 X2 Y1 Y2;

figure(4)

plot(time_int,rot_ang,'k-'), hold on

